@@ -30,7 +30,7 @@ enum {
     FILDES
 };
 
-int client(const struct optstruct *opts, int *infected);
+int client(const struct optstruct *opts, int *infected, int *err);
 int get_clamd_version(const struct optstruct *opts);
 int reload_clamd_database(const struct optstruct *opts);
 
