@@ -22,9 +22,16 @@
 
 #include "llvm/ADT/Triple.h"
 #include "llvm/Support/raw_ostream.h"
+#ifdef LLVM29
+#include "llvm/Support/Host.h"
+#include "llvm/Support/DataTypes.h"
+#include "llvm/Support/Memory.h"
+#else
 #include "llvm/System/Host.h"
 #include "llvm/System/DataTypes.h"
 #include "llvm/System/Memory.h"
+#endif
+
 #include "llvm/Config/config.h"
 
 extern "C" {
@@ -137,7 +144,9 @@ void cli_detect_env_jit(struct cli_environment *env)
 	CASE_OS(Linux, os_linux);
 	CASE_OS(Lv2, os_unknown);
 	CASE_OS(MinGW32, os_win32);
+#ifndef LLVM29
 	CASE_OS(MinGW64, os_win64);
+#endif
 	CASE_OS(NetBSD,  os_bsd);
 	CASE_OS(OpenBSD, os_bsd);
 	CASE_OS(Psp, os_unknown);
