@@ -133,7 +133,7 @@ static inline int hm_cmp(const uint8_t *itm, const uint8_t *ref, unsigned int ke
     return memcmp(&itm[4], &ref[4], keylen - 4);
 }
 
-void hm_sort(struct cli_sz_hash *szh, size_t l, size_t r, unsigned int keylen) {
+static void hm_sort(struct cli_sz_hash *szh, size_t l, size_t r, unsigned int keylen) {
     uint8_t piv[32], tmph[32];
     size_t l1, r1;
 
@@ -148,6 +148,7 @@ void hm_sort(struct cli_sz_hash *szh, size_t l, size_t r, unsigned int keylen) {
     while(l1 < r1) {
 	if(hm_cmp(&szh->hash_array[keylen * l1], piv, keylen) > 0) {
 	    r1--;
+	    if(l1 == r1) break;
 	    memcpy(tmph, &szh->hash_array[keylen * l1], keylen);
 	    tmpv = szh->virusnames[l1];
 	    memcpy(&szh->hash_array[keylen * l1], &szh->hash_array[keylen * r1], keylen);
@@ -218,7 +219,7 @@ int cli_hm_scan(const unsigned char *digest, uint32_t size, const char **virname
     keylen = hashlen[type];
 
     l = 0;
-    r = szh->items;
+    r = szh->items - 1;
     while(l <= r) {
 	size_t c = (l + r) / 2;
 	int res = hm_cmp(digest, &szh->hash_array[keylen * c], keylen);
