@@ -42,13 +42,10 @@ static	char	const	rcsid[] = "$Id: untar.c,v 1.35 2007/02/12 20:46:09 njh Exp $";
 #include "mbox.h"
 #include "blob.h"
 #include "scanners.h"
+#include "matcher.h"
 
 #define BLOCKSIZE 512
 
-#ifndef	O_BINARY
-#define	O_BINARY	0
-#endif
-
 static int
 octal(const char *str)
 {
@@ -80,7 +77,7 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 			if(fout>=0)
 				close(fout);
 			cli_errmsg("cli_untar: block read error\n");
-			return CL_EIO;
+			return CL_EREAD;
 		}
 
 		if(!in_block) {
@@ -89,12 +86,11 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 			char magic[7], name[101], osize[13];
 
 			if(fout>=0) {
-				int ret;
 				lseek(fout, 0, SEEK_SET);
 				ret = cli_magic_scandesc(fout, ctx);
 				close(fout);
-				if (!cli_leavetemps_flag)
-					unlink(fullname);
+				if (!ctx->engine->keeptmp)
+					if (cli_unlink(fullname)) return CL_EUNLINK;
 				if (ret==CL_VIRUS)
 					return CL_VIRUS;
 				fout = -1;
@@ -110,7 +106,7 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 				strncpy(magic, block+257, 5);
 				magic[5] = '\0';
 				if(strcmp(magic, "ustar") != 0) {
-					cli_dbgmsg("Incorrect magic string '%s' in tar header\n", magic);
+					cli_dbgmsg("cli_untar: Incorrect magic string '%s' in tar header\n", magic);
 					return CL_EFORMAT;
 				}
 			}
@@ -119,7 +115,7 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 
 			switch(type) {
 				default:
-					cli_warnmsg("cli_untar: unknown type flag %c\n", type);
+					cli_dbgmsg("cli_untar: unknown type flag %c\n", type);
 				case '0':	/* plain file */
 				case '\0':	/* plain file */
 				case '7':	/* contiguous file */
@@ -163,19 +159,21 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 			osize[12] = '\0';
 			size = octal(osize);
 			if(size < 0) {
-				cli_errmsg("Invalid size in tar header\n");
-				if(fout>=0)
-					close(fout);
-				return CL_CLEAN;
-			}
-			cli_dbgmsg("cli_untar: size = %d\n", size);
-			if((ret=cli_checklimits("cli_untar", ctx, size, 0, 0))!=CL_CLEAN) {
+				cli_dbgmsg("cli_untar: Invalid size in tar header\n");
 				skipEntry++;
+			} else {
+				cli_dbgmsg("cli_untar: size = %d\n", size);
+				if((ret=cli_checklimits("cli_untar", ctx, size, 0, 0))!=CL_CLEAN)
+					skipEntry++;
 			}
 
 			if(skipEntry) {
 				const int nskip = (size % BLOCKSIZE || !size) ? size + BLOCKSIZE - (size % BLOCKSIZE) : size;
-
+				
+				if(nskip < 0) {
+					cli_dbgmsg("cli_untar: got nagative skip size, giving up\n");
+					return CL_CLEAN;
+				}
 				cli_dbgmsg("cli_untar: skipping entry\n");
 				lseek(desc, nskip, SEEK_CUR);
 				continue;
@@ -183,15 +181,19 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 
 			strncpy(name, block, 100);
 			name[100] = '\0';
-			snprintf(fullname, sizeof(fullname)-1, "%s/tar%02u", dir, files);
+			if(cli_matchmeta(ctx, name, size, size, 0, files, 0, NULL) == CL_VIRUS)
+			    return CL_VIRUS;
+
+			snprintf(fullname, sizeof(fullname)-1, "%s"PATHSEP"tar%02u", dir, files);
 			fullname[sizeof(fullname)-1] = '\0';
 			fout = open(fullname, O_RDWR|O_CREAT|O_EXCL|O_TRUNC|O_BINARY, 0600);
 
 			if(fout < 0) {
-				cli_errmsg("Can't create temporary file %s: %s\n", fullname, strerror(errno));
+				char err[128];
+				cli_errmsg("cli_untar: Can't create temporary file %s: %s\n", fullname, cli_strerror(errno, err, sizeof(err)));
 				return CL_ETMPFILE;
 			}
-			
+
 			cli_dbgmsg("cli_untar: extracting to %s\n", fullname);
 
 			in_block = 1;
@@ -203,7 +205,7 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 				cli_errmsg("cli_untar: only wrote %d bytes to file %s (out of disc space?)\n",
 					nwritten, fullname);
 				close(fout);
-				return CL_EIO;
+				return CL_EWRITE;
 			}
 			size -= nbytes;
 		}
@@ -211,12 +213,11 @@ cli_untar(const char *dir, int desc, unsigned int posix, cli_ctx *ctx)
 			in_block = 0;
         }	
 	if(fout>=0) {
-		int ret;
 		lseek(fout, 0, SEEK_SET);
 		ret = cli_magic_scandesc(fout, ctx);
 		close(fout);
-		if (!cli_leavetemps_flag)
-			unlink(fullname);
+		if (!ctx->engine->keeptmp)
+			if (cli_unlink(fullname)) return CL_EUNLINK;
 		if (ret==CL_VIRUS)
 			return CL_VIRUS;
 	}
