@@ -30,7 +30,8 @@
 #define OPT_SIGTOOL	32
 #define OPT_CLAMCONF	64
 #define OPT_CLAMDTOP	128
-#define OPT_DEPRECATED	256
+#define OPT_CLAMBC      256
+#define OPT_DEPRECATED	512
 
 #define TYPE_STRING  1	/* quoted/regular string */
 #define TYPE_NUMBER  2	/* raw number */
@@ -66,8 +67,6 @@ struct clam_option {
     const char *suggested;
 };
 
-extern const struct clam_option clam_options[];
-
 const struct optstruct *optget(const struct optstruct *opts, const char *name);
 
 void optfree(struct optstruct *opts);
